#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "person.h"
#include "personio.h"


void swap_persons(person_t *jack, person_t *jill)
{
    person_t  t;

    t = *jack;  *jack = *jill;  *jill = t;
} /* swap_persons */


void sort_persons_id(person_t* ps, int len)
{
    int sorted;    /* already sorted until this index */
    int tail_mini; /* Index of minimum from sorted to the end */

    sorted = 0;    /* Nothing is sorted yet */
    while (sorted < len)
    {
        int  j;

        tail_mini = sorted;
        j = tail_mini + 1;
        while (j < len)
        {
            if (ps[tail_mini].numeric_id > ps[j].numeric_id)
            {
                tail_mini = j;
            }
            j++;
        }

        /* swap so a[sorted] will get the minimum of the tail */
        swap_persons(&ps[sorted], &ps[tail_mini]);
        sorted++;
    }
} /* sort_persons_id */


void sort_persons_name(person_t* ps, int len)
{
    int sorted;    /* already sorted until this index */
    int tail_mini; /* Index of minimum from sorted to the end */

    sorted = 0;    /* Nothing is sorted yet */
    while (sorted < len)
    {
        int  j;

        tail_mini = sorted;
        j = tail_mini + 1;
        while (j < len)
        {
            if (strcmp(ps[tail_mini].name, ps[j].name) > 0)
            {
                tail_mini = j;
            }
            j++;
        }

        /* swap so a[sorted] will get the minimum of the tail */
        swap_persons(&ps[sorted], &ps[tail_mini]);
        sorted++;
    }
} /* sort_persons_name */


void sort_persons_weight(person_t* ps, int len)
{
    int sorted;    /* already sorted until this index */
    int tail_mini; /* Index of minimum from sorted to the end */

    sorted = 0;    /* Nothing is sorted yet */
    while (sorted < len)
    {
        int  j;

        tail_mini = sorted;
        j = tail_mini + 1;
        while (j < len)
        {
            if (ps[tail_mini].weight > ps[j].weight)
            {
                tail_mini = j;
            }
            j++;
        }

        /* swap so a[sorted] will get the minimum of the tail */
        swap_persons(&ps[sorted], &ps[tail_mini]);
        sorted++;
    }
} /* sort_persons_weight */


void sort_persons(person_t* ps, int np, sort_key_t sort_key)
{
    /* Expression inside switch (parentheis) must be  int  or  enum value */
    switch (sort_key)
    {
     /* Cases must be given _constants_ ! */
     case SORT_KEY_ID:
        sort_persons_id(ps, np);
        break;
     case SORT_KEY_NAME:
        sort_persons_name(ps, np);
        break;
     case SORT_KEY_WEIGHT:
        sort_persons_weight(ps, np);
        break;
    }
} /* sort_persons */


int usage(char *progname)
{
    fprintf(stderr,
        "Sort persons database via key (id, name or weight)\n"
        "\n"
        "Usage:\n"
        "  %s [-sortid | -sortnm | -sortw] [db-input] [report-output]\n"
        "\n"
        "Where default key is 'id'\n", progname);
    return 1;
} /* usage */


int get_parameters(int argc, char **argv,
    sort_key_t *p_sort_key,
    char **p_fn_db,
    char **p_fn_rep
)
{
    int         rc = 0;
    sort_key_t  sort_key = SORT_KEY_ID;

    if (argc < 3)
    {
         rc = usage(argv[0]);
    }
    else
    {
         int   ai = 1;
         char  *opt = argv[ai];

         if (strcmp(opt, "-sortid") == 0)
         {
             sort_key = SORT_KEY_ID;      ai++;
         }
         else if (strcmp(opt, "-sortnm") == 0)
         {
             sort_key = SORT_KEY_NAME;    ai++;
         }
         else if (strcmp(opt, "-sortw") == 0)
         {
             sort_key = SORT_KEY_WEIGHT;  ai++;
         }

         if (ai + 2 != argc)
         {
             rc = usage(argv[0]);
         }
         else
         {
             *p_fn_db = argv[ai];
             *p_fn_rep = argv[ai + 1];
         }
    }
    *p_sort_key = sort_key;
    return rc;
} /* get_parameters */


int main(int argc, char **argv)
{
    int         rc = 0;
    char        *fn_db;
    char        *fn_rep;
    person_t    *persons = NULL;
    int         n_persons = 0;
    sort_key_t  sort_key = SORT_KEY_ID;

    rc = get_parameters(argc, argv, &sort_key, &fn_db, &fn_rep);
    if (rc == 0)
    {
        n_persons = read_count_persons(fn_db, &persons);
        if (n_persons < 0)
        {
            rc = 1;
        }
        else
        {
            sort_persons(persons, n_persons, sort_key);
            persons_report(fn_rep, persons, n_persons);
        }
    }
    if (persons != NULL) { free(persons); }
    return rc;
}
