#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "person.h"
#include "personio.h"


/* Parse line for personal data, and enter it into person.
 * Return 1 iff success, 0 otherwise
 */
int add_person(person_t* person, char *line)
{
    char   *pline, *colon;

    colon = strchr(line, ':');
    if (!colon)
    {
        fprintf(stderr, "Bad line: %s\n", line);
        return 0;
    }
    *colon = '\0'; /* null terminate the name */
    strcpy(person->name, line);

    pline = colon + 1;
    *colon = '\0';   /* null terminate the id */
    person->numeric_id = strtol(pline, 0, 0);
    colon = strchr(pline, ':');
    if (!colon)
    {
        fprintf(stderr, "Bad line: %s\n", line);
        return 0;
    }

    pline = colon + 1;
    person->weight = strtod(pline, 0);
    return 1;
} /* add_person */


int read_count_persons(char *fn, person_t **persons_p)
{
    int       n_persons = 0;
    FILE      *f = fopen(fn, "r");
    char      line[0x100]; /* == 256 */
    person_t  *persons = NULL;

    if (!f)
    {
        fprintf(stderr, "Failed to open %s\n", fn);
        n_persons = -1;
    }

    while ((0 <= n_persons) && fgets(line, sizeof(line), f))
    {
        if (line[0] != '#')
        {
            n_persons++;
            /* With persons=NULL, 1st realloc() call, will be like malloc() */
	    persons = realloc(persons, n_persons * sizeof(person_t));
            add_person(&persons[n_persons - 1], line);
        }
    }

    if (f) { fclose(f); }
    *persons_p = persons; /* Caller should receive the new pointer! */
    return n_persons;
} /* read_persons */


void person_print(FILE *f, person_t *p)
{
    fprintf(f, "%-20s  id:%8d   %.3fKg\n", p->name, p->numeric_id, p->weight);
} /* person_print */


void persons_report(char *fn, person_t *ps, int np)
{
    int    i;
    FILE*  f;

    f = fopen(fn, "w");
    for (i = 0;  i < np;  i++)
    {
        person_print(f, &ps[i]);
    }
    fclose(f);
} /* persons_report */
